 /*********************************************************************
 *
 *                  MCP23017 Library definitions
 *         			16-Bit IO Expander
 *
 *********************************************************************
 * FileName:        mcp23017_lib.c
 * Dependencies:
 * Processor:       PIC32
 *
 * Complier:        MPLAB C32
 *                  MPLAB IDE
 * Author:			Carl
 ********************************************************************/
#include <plib.h>
#include "C:\Microchip Solutions\Microchip\Include\GenericTypeDefs.h"
//*****************************************************
//	I2C2 Issue on PIC32 library function	IdleI2C2();
//
//	Use this instead of library because of a typo in the library call IdleI2C2();
//
//		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
//		IdleI2C2();		//bug in this library code, use with above line for I2C2
//
//	 IdleI2C1(); works just fine for I2C1
//
//***********************************************************************/

char MCP23017_readRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg);
void MCP23017_writeRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);
void MCP23017_setBANK_1(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char ConfigData);
void ClearRegisterBit(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);
void SetRegisterBit(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);

char MCP23017_I2C1_readRegister(unsigned char SlaveAddr, unsigned char Reg);
void MCP23017_I2C1_writeRegister(unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);
void MCP23017_I2C1_setBANK_1(unsigned char SlaveAddr, unsigned char ConfigData);

char MCP23017_I2C2_readRegister(unsigned char SlaveAddr, unsigned char Reg);
void MCP23017_I2C2_writeRegister(unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);
void MCP23017_I2C2_setBANK_1(unsigned char SlaveAddr, unsigned char ConfigData);

//***********************************************************************/
//	Useful information for IOCON setup
//
// Set up IOCON register, assume BANK=0 from power up, add reset capability later
// BANK = 1                 <<===ICON.BANK = 1 now
// MIRROR = 1
// SEQOP = 1
// DISSLW = 0
// HAEN = 1   NOT USED
// ODR = 1
// INTPOL = 0  over ridden by ODR=1
// 0
//  the above then:    IOCON = 0xEC
//
/******************************************************************************
 * Function:        void MCP23017_I2C1_setBANK_1(unsigned char SlaveAddr)
 *
 * Description:		Sets IOCON BANK=1 for MCP23017 on I2C1, regardless of current
 *					bank setting.  Try writting a '1' to bit 0 of the IOCON register
 *					If correctly addressed the IOCON register will always return a '0'.
 *					If not in correct bank then restore register to orginal value
 *					and try other bank to insure BANK=1 and with the other correct
 *					configuration bits desired when done.
 *
 * PreCondition:    None
 *
 * Inputs:	 		None
 *					
 * Output:          None
 *
 * Example:			MCP23017_I2C1_setBANK_1(48);
 *
 *****************************************************************************/
void MCP23017_setBANK_1(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char ConfigData)
{
	if (I2Cport == 1)
	{
		MCP23017_I2C1_setBANK_1(SlaveAddr, ConfigData);
	}
		else
		{
			if (I2Cport == 2)
			{
				MCP23017_I2C2_setBANK_1(SlaveAddr, ConfigData);
			}
		}
}

void MCP23017_writeRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data)
{
	if (I2Cport == 1)
	{
		MCP23017_I2C1_writeRegister(SlaveAddr, Reg, Data);
	}
		else
		{
			if (I2Cport == 2)
			{
				MCP23017_I2C2_writeRegister(SlaveAddr, Reg, Data);
			}
		}
}

char MCP23017_readRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg)
{
	if (I2Cport == 1)
	{
		MCP23017_I2C1_readRegister(SlaveAddr, Reg);
	}
		else
		{
			if (I2Cport == 2)
			{
				MCP23017_I2C2_readRegister(SlaveAddr, Reg);
			}
		}
}

void MCP23017_I2C1_setBANK_1(unsigned char SlaveAddr, unsigned char ConfigData)
{
	char i2cData[3], ReferenceData, TestData, BANK;
	int DataSz;
	int Index;
//get reference data at test location 0x0A, which might be the IOCON register or not
	ReferenceData = MCP23017_I2C1_readRegister(SlaveAddr, 0x0A); 
	MCP23017_I2C1_writeRegister(SlaveAddr, 0x0A, 0x01);
	TestData = MCP23017_I2C1_readRegister(SlaveAddr, 0x0A);
	if (TestData == 0x00)
	{
		BANK = 0;
	}
	else 
		BANK = 1;		// Register 0x0A would actually be the OLATA register in BANK=1

	if (BANK==0)
	{
		MCP23017_I2C1_writeRegister(SlaveAddr, 0x0A, ConfigData);  //0x0A IOCON in BANK=0
	}
	else
	{
		MCP23017_I2C1_writeRegister(SlaveAddr, 0x0A, ReferenceData);  //Restore ReferenceData
		MCP23017_I2C1_writeRegister(SlaveAddr, 0x05, ConfigData);  //0x05 IOCON in BANK=1
	}
}

//===============================================================
//
//					  BANK=1 now
//
//================================================================
//
//					I2C1
//
//================================================================
//   Read one register in the MCP23017
char MCP23017_I2C1_readRegister(unsigned char SlaveAddr, unsigned char Reg)
{ 
	char i2cData[2];
	int DataSz;
	int Index;
// Set up to Read register
	i2cData[0] = SlaveAddr;			//Slave Address, Device address and R Command
 	i2cData[1] = Reg;				//Set to register to read

	DataSz = 2;
	StartI2C1();	//Send the Start Bit
   	IdleI2C1();		//Wait to complete
	Index = 0;
 	while(DataSz)
 	{
	MasterWriteI2C1( i2cData[Index++] );
 	IdleI2C1();//Wait to complete

	DataSz--;
 	}

// Send READ command
 	i2cData[0] = (SlaveAddr |1);	//Slave Address, Device address and R Command
 	i2cData[1] = 0x00;				//read data goes here

	DataSz = 1;
	Index = 0;
 	RestartI2C1();
	IdleI2C1();		//Wait to complete

 while(DataSz)
	{
	MasterWriteI2C1( i2cData[Index++] );
 	IdleI2C1();		//Wait to complete

	DataSz--;
	}
 	i2cData[1] = MasterReadI2C1();
	StopI2C1();		//Send the Stop condition
 	IdleI2C1();		//Wait to complete

	Index = i2cData[1];   //for debug only
	return (i2cData[1]);
}


void MCP23017_I2C1_writeRegister(unsigned char SlaveAddr, unsigned char Reg, unsigned char Data)
{
	char i2cData[3];
	int DataSz;
	int Index;
 
	i2cData[0] = SlaveAddr;			//Slave Address, Device address and WR Command
 	i2cData[1] = Reg;				//Register to write to
	i2cData[2] = Data;				//Data to write

	DataSz = 3;
	StartI2C1();	//Send the Start Bit
  	IdleI2C1();		//Wait to complete

	Index = 0;
	while(DataSz)
	{
	MasterWriteI2C1( i2cData[Index++] );
 	IdleI2C1();		//Wait to complete

	DataSz--;
	}

	StopI2C1();								//Send the Stop condition
	IdleI2C1()								;//Wait to complete
	return;
}

//==============================================================================
//==============================================================================
//
//			I2C2 port calls
//
//==============================================================================
void MCP23017_I2C2_setBANK_1(unsigned char SlaveAddr, unsigned char ConfigData)
{
	char i2cData[3], ReferenceData, TestData, BANK;
	int DataSz;
	int Index;
//get reference data at test location 0x0A, which might be the IOCON register or not
	ReferenceData = MCP23017_I2C2_readRegister(SlaveAddr, 0x0A); 
	MCP23017_I2C2_writeRegister(SlaveAddr, 0x0A, 0x01);
	TestData = MCP23017_I2C2_readRegister(SlaveAddr, 0x0A);
	if (TestData == 0x00)
	{
		BANK = 0;
	}
	else 
		BANK = 1;		// Register 0x0A would actually be the OLATA register in BANK=1

	if (BANK==0)
	{
		MCP23017_I2C2_writeRegister(SlaveAddr, 0x0A, ConfigData);  //0x0A IOCON in BANK=0
	}
	else
	{
		MCP23017_I2C2_writeRegister(SlaveAddr, 0x0A, ReferenceData);  //Restore ReferenceData
		MCP23017_I2C2_writeRegister(SlaveAddr, 0x05, ConfigData);  //0x05 IOCON in BANK=1
	}
}

//===============================================================
//
//					  BANK=1 now

//================================================================
//
//   Read one register in the MCP23017
char MCP23017_I2C2_readRegister(unsigned char SlaveAddr, unsigned char Reg)
{ 
	char i2cData[2];
	int DataSz;
	int Index;
// Set up to Read register
	i2cData[0] = SlaveAddr;			//Slave Address, Device address and R Command
 	i2cData[1] = Reg;				//Set to register to read

	DataSz = 2;
	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	Index = 0;
 	while(DataSz)
 	{
	MasterWriteI2C2( i2cData[Index++] );
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	DataSz--;
 	}

// Send READ command
 	i2cData[0] = (SlaveAddr |1);	//Slave Address, Device address and R Command
 	i2cData[1] = 0x00;				//read data goes here

	DataSz = 1;
	Index = 0;
 	RestartI2C2();
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

 while(DataSz)
	{
	MasterWriteI2C2( i2cData[Index++] );
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	DataSz--;
	}
 	i2cData[1] = MasterReadI2C2();
	StopI2C2();		//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	Index = i2cData[1];   //for debug only
	return (i2cData[1]);
}


void MCP23017_I2C2_writeRegister(unsigned char SlaveAddr, unsigned char Reg, unsigned char Data)
{
	char i2cData[3];
	int DataSz;
	int Index;
 
	i2cData[0] = SlaveAddr;			//Slave Address, Device address and WR Command
 	i2cData[1] = Reg;				//Register to write to
	i2cData[2] = Data;				//Data to write

	DataSz = 3;
	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	Index = 0;
	while(DataSz)
	{
	MasterWriteI2C2( i2cData[Index++] );
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	DataSz--;
	}

	StopI2C2();								//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	return;
}

//===========================================

// clear Register bit
void ClearRegisterBit(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data)
{
	BYTE CurrentData;
// read register first
CurrentData = MCP23017_readRegister(I2Cport, SlaveAddr, Reg);
CurrentData = (Data ^ (0xFF)) & CurrentData;	//bitwise XOR
MCP23017_writeRegister(I2Cport, SlaveAddr, Reg, CurrentData);
}

// set Register bit
void SetRegisterBit(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data)
{
	BYTE CurrentData;
// read register first
CurrentData  = MCP23017_readRegister(I2Cport, SlaveAddr, Reg);
CurrentData = (CurrentData | Data);
MCP23017_writeRegister(I2Cport, SlaveAddr, Reg, CurrentData);
}

