/*********************************************************************
 *
 *                  MCP9800 Library definitions
 *         9-bit to 12-bit I2C High-Accuracy Temperature Sensor
 *
 *********************************************************************
 * FileName:        mcp9800.h
 * Dependencies:
 * Processor:       PIC32
 *
 * Complier:        MPLAB C32
 *                  MPLAB IDE
 * Author:			Carl
 ********************************************************************/
#include <plib.h>
#include "BMS_HardwareConfig.h"		//Harware configuration for BMS project
//
//*****************************************************
//	I2C2 Issue on PIC32 library function	IdleI2C2();
//
//	Use this instead of library because of a typo in the library call IdleI2C2();
//
//		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
//		IdleI2C2();		//bug in this library code, use with above line for I2C2
//
//	 IdleI2C1(); works just fine for I2C1
//
//***********************************************************************/
//
//#define SYSCLK	(80000000)
//#define PBCLK  (SYSCLK/2)
//#define Fsck	50000
//#define BRG_VAL 	((PBCLK/2/Fsck)-2)
/**************************************************************
 * config Parameter Values to be used with MCP9800_I2C1_configure()
 *************************************************************/
#define ONE_SHOT_DISABLE	(0)			//One-Shot Disabled (power-up default)
#define ONE_SHOT_ENABLE 	(1<<7)		//One-Shot Enabled
#define RES_9BIT			(0)			//ADC Resolution 9bit (power-up default)
#define RES_10BIT 			(1<<5)		//ADC Resolution 10bit
#define RES_11BIT			(2<<5)		//ADC Resolution 11bit
#define RES_12BIT 			(3<<5)		//ADC Resolution 12bit
#define FAULT_QUEUE_1		(0)			//Fault Queue 1 (power-up default) 
#define FAULT_QUEUE_2		(1<<3)		//Fault Queue 2
#define FAULT_QUEUE_4		(2<<3)		//Fault Queue 4
#define FAULT_QUEUE_6		(3<<3)		//Fault Queue 6
#define ALERT_POLARITY_LOW	(0)			//Alert Polarity Low (power-up default)
#define ALERT_POLARITY_HIGH	(1<<2)		//Alert Polarity High
#define COMPARE_MODE 		(0)			//Compare Mode (power-up default)
#define INTERRUPT_MODE		(1<<1)		//Interrupt Mode
#define SHUTDOWN_DISABLE	(0)			//Shutdown Disable	(power-up default)
#define SHUTDOWN_ENABLE		(1)			//Shutdown Enabled	
//******************************************
//#define Mcp9800SlaveAddr   0x90;

void MCP9800_configure(unsigned char I2Cport, unsigned char DeviceAddress, unsigned char Config);
int	MCP9800_ReadTemperature(unsigned char I2Cport, unsigned char DeviceAddress);
int Mcp9800_FormattedRead(unsigned char I2Cport, unsigned char DeviceAddress);

void MCP9800_I2C1_configure(unsigned char DeviceAddress, unsigned char Config);
int	MCP9800_I2C1_ReadTemperature(unsigned char DeviceAddress);
int Mcp9800_I2C1_FormattedRead(unsigned char DeviceAddress);

void MCP9800_I2C2_configure(unsigned char DeviceAddress, unsigned char Config);
int	MCP9800_I2C2_ReadTemperature(unsigned char DeviceAddress);
int Mcp9800_I2C2_FormattedRead(unsigned char DeviceAddress);

//==============================================================================

#define Nop() asm( "nop" )

void i2c_wait(unsigned int cnt)
{
	while(--cnt)
	{
		Nop();
		Nop();
	}
}
//
// MCP9801 register pointer
// bit 7-3 unused, set to 0
// bit 2-0 Pointer bits
//		00 = Temperature Register
//		01 = Configuration Register
//		10 = Temperature Hysteresis
//		11 = Temperature Limit-set Register
//
void MCP9800_configure(unsigned char I2Cport, unsigned char DeviceAddress, unsigned char Config)
{
	if (I2Cport == 1)
	{
		MCP9800_I2C1_configure(DeviceAddress, Config);
	}
		else
		{
			if (I2Cport == 2)
			{
				MCP9800_I2C2_configure(DeviceAddress, Config);
			}
		}
}

int	MCP9800_ReadTemperature(unsigned char I2Cport, unsigned char DeviceAddress)
{
	if (I2Cport == 1)
	{
		MCP9800_I2C1_ReadTemperature(DeviceAddress);
	}
		else
		{
			if (I2Cport == 2)
			{
				MCP9800_I2C2_ReadTemperature(DeviceAddress);
			}
		}
}

int Mcp9800_FormattedRead(unsigned char I2Cport, unsigned char DeviceAddress)
{
	if (I2Cport == 1)
	{
		Mcp9800_I2C1_FormattedRead(DeviceAddress);
	}
		else
		{
			if (I2Cport == 2)
			{
				Mcp9800_I2C2_FormattedRead(DeviceAddress);
			}
		}
}

//============================================================================================
/******************************************************************************
 * Function:        void MCP9800_I2C1_configure(unsigned char DeviceAddress, unsigned char config)
 *
 * Description:		Sets Configuration Register for MCP9800 on I2C1
 *
 * PreCondition:    None
 *
 * Inputs:			Bit-wise OR value of 
 *					(ONE_SHOT_DISABLE || ONE_SHOT_ENABLE),
 *					(RES_9BIT || RES_10BIT || RES_11BIT || RES_12BIT),
 *					(FAULT_QUEUE_1 || FAULT_QUEUE_2 || FAULT_QUEUE_4 || FAULT_QUEUE_6),
 *					(ALERT_POLARITY_LOW || ALERT_POLARITY_HIGH),
 *					(COMPARE_MODE || INTERRUPT_MODE),
 *					(SHUTDOWN_DISABLE || SHUTDOWN_ENABLE)
 *
 *					An absent symbol is automatically taken as off or default value.
 *
 * Output:          None
 *
 * Example:			MCP9800_I2C1_configure(ONE_SHOT_DISABLE | RES_12BIT
 *									 | ALERT_POLARITY_HIGH | COMPARE_MODE);
 *
 *****************************************************************************/
void MCP9800_I2C1_configure(unsigned char DeviceAddress, unsigned char Config)
{
	char i2cData[3];
	int DataSz;
	
	i2cData[0] = 0x90 | (DeviceAddress <<1 | 0);	//Slave Address, Device address and WR Command
	i2cData[1] = 0x01;								//Set to Pointer Register
	i2cData[2] = Config;							//Configuration data
	DataSz = 3;

	StartI2C1();	//Send the Start Bit
	IdleI2C1();		//Wait to complete

	int Index = 0;
	while(DataSz)
	{
	MasterWriteI2C1( i2cData[Index++] );
	IdleI2C1();//Wait to complete

	DataSz--;
	//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
	if( I2C1STATbits.ACKSTAT )
	break;
	}

	StopI2C1();								//Send the Stop condition
	IdleI2C1()								;//Wait to complete
}


/******************************************************************************
 * Function:        int MCP9800_I2C1_ReadTemperature(unsigned char DeviceAddress, unsigned char config)
 *
 * Description:		Reads Temperature Register for MCP9800 on I2C1
 *
 * PreCondition:    None
 *
 * Inputs:			None 
 *
 * Output:          Temperature
 *
 * Example:			Temperature = MCP9800_I2C1_ReadTemperature(DeviceAddress);
 *
 *****************************************************************************/
int	MCP9800_I2C1_ReadTemperature(unsigned char DeviceAddress)
{
	char i2cData[3];
	int DataSz;
	
	i2cData[0] = 0x90 | (DeviceAddress <<1 | 0);	//Slave Address, Device address and WR Command
	i2cData[1] = 0x00;								//Set to Temperature Register
	DataSz = 2;

	StartI2C1();	//Send the Start Bit
	IdleI2C1();		//Wait to complete

	int Index = 0;
	while(DataSz)
	{
	MasterWriteI2C1( i2cData[Index++] );
	IdleI2C1();//Wait to complete

	DataSz--;
	//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
	if( I2C1STATbits.ACKSTAT )
	break;
	}

	StopI2C1();								//Send the Stop condition
	IdleI2C1()								;//Wait to complete

	i2cData[0] = 0x90 | (DeviceAddress <<1 | 1);	//Slave Address, Device address and R Command
	StartI2C1();
	IdleI2C1();//Wait to complete

	MasterWriteI2C1( i2cData[0] ); //transmit read command
	IdleI2C1();//Wait to complete
	
	i2cData[1] = MasterReadI2C1();
	AckI2C1();

 	IdleI2C1();//Wait to complete

	i2cData[2] = MasterReadI2C1();
	NotAckI2C1();

	StopI2C1();								//Send the Stop condition
	IdleI2C1()								;//Wait to complete
	return (i2cData[1]<<8) + i2cData[2];
}

//=============================================================
//Formatted read
int Mcp9800_I2C1_FormattedRead(unsigned char DeviceAddress)
{
	unsigned char TemperatureStr[10];
	unsigned int TemperatureLeftPart, TemperatureRightPart;
	int Temperature, TemperatureFrac;

	NotAckI2C1();
	CloseI2C1();	
	OpenI2C1( I2C_EN, BRG_VAL );		//Enable channel

	MCP9800_I2C1_configure(DeviceAddress, 0x20);	//0x60 = 12-bit, 0x40 = 11-bit, 20=10bit, 00=9bit
	Temperature = MCP9800_I2C1_ReadTemperature(DeviceAddress);	//Read 12-bit format

	CloseI2C1();

// 12-bit format is as follows:
// 16-bit integer has left byte as whole degrees C
// The right byte is 
// bit 8	0.5 C
// bit 7	0.25 C
// bit 6	0.125 C
// bit 5	0.0625 C

TemperatureLeftPart =(Temperature>>8);	//Whole degrees are saved in the left byte
TemperatureRightPart = (Temperature & 0xFF);	//Fraction degrees are in the right byte

TemperatureFrac = 0;					//Clear the fraction part to zero

if (!(Temperature & 0x8000))			//test for positive value
	{
		if (TemperatureRightPart & 0x80)
		{
			TemperatureFrac = TemperatureFrac + 5000;
		}
		if (TemperatureRightPart & 0x40)
		{
			TemperatureFrac = TemperatureFrac + 2500;
		}
		if (TemperatureRightPart & 0x20)
		{
			TemperatureFrac = TemperatureFrac + 1250;
		}
		if (TemperatureRightPart & 0x10)
		{
			TemperatureFrac = TemperatureFrac + 625;
		}
	}
	else		//negative temperature, this needs to be implemented yet
	{
TemperatureLeftPart = 0;
TemperatureRightPart = 0;
	}

return	(Temperature);
}

//======================================================================
//
//		I2C2 port
//
//======================================================================
/******************************************************************************
 * Function:        void MCP9800_I2C2_configure(unsigned char DeviceAddress, unsigned char config)
 *
 * Description:		Sets Configuration Register for MCP9800 on I2C2
 *
 * PreCondition:    None
 *
 * Inputs:			Bit-wise OR value of 
 *					(ONE_SHOT_DISABLE || ONE_SHOT_ENABLE),
 *					(RES_9BIT || RES_10BIT || RES_11BIT || RES_12BIT),
 *					(FAULT_QUEUE_1 || FAULT_QUEUE_2 || FAULT_QUEUE_4 || FAULT_QUEUE_6),
 *					(ALERT_POLARITY_LOW || ALERT_POLARITY_HIGH),
 *					(COMPARE_MODE || INTERRUPT_MODE),
 *					(SHUTDOWN_DISABLE || SHUTDOWN_ENABLE)
 *
 *					An absent symbol is automatically taken as off or default value.
 *
 * Output:          None
 *
 * Example:			MCP9800_I2C2_configure(ONE_SHOT_DISABLE | RES_12BIT
 *									 | ALERT_POLARITY_HIGH | COMPARE_MODE);
 *
 *****************************************************************************/
void MCP9800_I2C2_configure(unsigned char DeviceAddress, unsigned char Config)
{
	char i2cData[3];
	int DataSz;
	
	i2cData[0] = 0x90 | (DeviceAddress <<1 | 0);	//Slave Address, Device address and WR Command
	i2cData[1] = 0x01;								//Set to Pointer Register
	i2cData[2] = Config;							//Configuration data
	DataSz = 3;

	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	int Index = 0;
	while(DataSz)
	{
	MasterWriteI2C2( i2cData[Index++] );
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	DataSz--;
	//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
	if( I2C2STATbits.ACKSTAT )
	break;
	}

	StopI2C2();								//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
}


/******************************************************************************
 * Function:        int MCP9800_I2C2_ReadTemperature(unsigned char DeviceAddress, unsigned char config)
 *
 * Description:		Reads Temperature Register for MCP9800 on I2C2
 *
 * PreCondition:    None
 *
 * Inputs:			None 
 *
 * Output:          Temperature
 *
 * Example:			Temperature = MCP9800_I2C2_ReadTemperature(DeviceAddress);
 *
 *****************************************************************************/
int	MCP9800_I2C2_ReadTemperature(unsigned char DeviceAddress)
{
	char i2cData[3];
	int DataSz;
	
	i2cData[0] = 0x90 | (DeviceAddress <<1 | 0);	//Slave Address, Device address and WR Command
	i2cData[1] = 0x00;								//Set to Temperature Register
	DataSz = 2;

	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	int Index = 0;
	while(DataSz)
	{
	MasterWriteI2C2( i2cData[Index++] );
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	DataSz--;
	//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
	if( I2C2STATbits.ACKSTAT )
	break;
	}

	StopI2C2();								//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	i2cData[0] = 0x90 | (DeviceAddress <<1 | 1);	//Slave Address, Device address and R Command
	StartI2C2();
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	MasterWriteI2C2( i2cData[0] ); //transmit read command
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	
	i2cData[1] = MasterReadI2C2();
	AckI2C2();

	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	i2cData[2] = MasterReadI2C2();
	NotAckI2C2();

	StopI2C2();								//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	return (i2cData[1]<<8) + i2cData[2];
}

//=============================================================
//Formatted read
int Mcp9800_I2C2_FormattedRead(unsigned char DeviceAddress)
{
	unsigned char TemperatureStr[10];
	unsigned int TemperatureLeftPart, TemperatureRightPart;
	int Temperature, TemperatureFrac;

	NotAckI2C2();
	CloseI2C2();	
	OpenI2C2( I2C_EN, BRG_VAL );		//Enable channel

	MCP9800_I2C2_configure(DeviceAddress, 0x60);	//0x60 = 12-bit, 0x40 = 11-bit, 20=10bit, 00=9bit
	Temperature = MCP9800_I2C2_ReadTemperature(DeviceAddress);	//Read 12-bit format

	CloseI2C2();

// 12-bit format is as follows:
// 16-bit integer has left byte as whole degrees C
// The right byte is 
// bit 8	0.5 C
// bit 7	0.25 C
// bit 6	0.125 C
// bit 5	0.0625 C

TemperatureLeftPart =(Temperature>>8);	//Whole degrees are saved in the left byte
TemperatureRightPart = (Temperature & 0xFF);	//Fraction degrees are in the right byte

TemperatureFrac = 0;					//Clear the fraction part to zero

if (!(Temperature & 0x8000))			//test for positive value
	{
		if (TemperatureRightPart & 0x80)
		{
			TemperatureFrac = TemperatureFrac + 5000;
		}
		if (TemperatureRightPart & 0x40)
		{
			TemperatureFrac = TemperatureFrac + 2500;
		}
		if (TemperatureRightPart & 0x20)
		{
			TemperatureFrac = TemperatureFrac + 1250;
		}
		if (TemperatureRightPart & 0x10)
		{
			TemperatureFrac = TemperatureFrac + 625;
		}
	}
	else		//negative temperature, this needs to be implemented yet
	{
TemperatureLeftPart = 0;
TemperatureRightPart = 0;
	}

return	(Temperature);
}
