//=============================================
//
//	BMS_Entrance_Servo.c
//
//=============================================
//
#include <p32xxxx.h>
#include <plib.h>
#include "GenericTypeDefs.h"
#include <TCPIP Stack/Tick.h>


void Set_New_Entrance_Opening(unsigned short int Start, unsigned short int Stop);
//======================================================================================================

// set opening of hive
// 0x0E80 = ~1.5mS pulse width, 0x0800 = 1mS
// 0x0700 = full ccw, 0x0E00= mid, 0x1500= full cw  
//Validate_Entrance_Opening(0x10,  0x1500);		//sets PWM

extern BYTE ServoBusyFlag;		//TRUE Indicates a servo is still being positioned, new position cmd can not be accepted yet

void Validate_Entrance_Opening(BYTE ServoNum, unsigned short int Start, unsigned short int Stop)
{
// BMS Global variables
//These valuesd are used to program the PWM for each servo's position
extern unsigned short int	Previous_Servo_Start[8];
extern unsigned short int	Previous_Servo_Stop[8];


unsigned short int Current_Servo_Start;
unsigned short int Current_Servo_Stop;


Current_Servo_Start = Start;
Current_Servo_Stop = Stop;
// set new pulse width if necessary
if ((Current_Servo_Start == Previous_Servo_Start[ServoNum]) & (Current_Servo_Stop == Previous_Servo_Stop[ServoNum]))
	return;	// new setting was not necessary, keep old

// update to new setting
Previous_Servo_Start[ServoNum] = Current_Servo_Start;
Previous_Servo_Stop[ServoNum] = Current_Servo_Stop;
Set_New_Entrance_Opening(Current_Servo_Start, Current_Servo_Stop);
//Now turn Servo Power on for a short time to run
if (ServoNum == 1)
		MCP23017_writeRegister(AuxioPort, AuxioSA, 0x09, 0x01);	//set GPIOA bit to turn servo pwr on
if (ServoNum == 2)
		MCP23017_writeRegister(AuxioPort, AuxioSA, 0x09, 0x02);	//set GPIOA bit to turn servo pwr on
return;
}

void Set_New_Entrance_Opening(unsigned short int Start, unsigned short int Stop)
{
//static TICK tServo = 0;
extern DWORD tgServo;		//used to make sure servo has time to move to new position before turning power off to it

if (TickGet() - tgServo < TICK_SECOND)
	return;
tgServo = TickGet();
		MCP23017_writeRegister(AuxioPort, AuxioSA, 0x09, 0x00);	//set GPIOA bit to turn servo pwr on

 OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_16, 0xF000);  //~25mS period

/* Holds the value at which OCx Pin to be driven high */
unsigned int pulse_start ;
/* Holds the value at which OCx Pin to be driven low */
unsigned int pulse_stop;
/* Turn off OC3 module */
CloseOC3();
/* Configure OC3 module for required pulse width */
pulse_start = Start;
pulse_stop = Stop;		// 0x0E80 = ~1.5mS pulse width, 0x0800 = 1mS
							// 0x0700 = full ccw, 0x0E00= mid, 0x1500= full cw  
// PR2 = 0x0FFF ;
// T2CON = 0x8000;
/* Configure Output Compare module to 'initialise OC3 pin for continuous pulse'mode */
OpenOC3(OC_ON | OC_IDLE_CON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE, pulse_stop, pulse_start);
}

void EntranceServo1_Open(void)		//Validate that Entrance servo 1 is Open 
{
Validate_Entrance_Opening(1, 0x10,  0x2000);	//check servo 1 position
}

void EntranceServo1_Closed(void)		//Validate that Entrance servo 1 is Closed
{
Validate_Entrance_Opening(1, 0x10,  0x1000);	//check servo 1 position
}

void EntranceServo2_Open(void)		//Validate that Entrance servo 2 is Open 
{
Validate_Entrance_Opening(2, 0x10,  0x2000);	//check servo 2 position
}

void EntranceServo2_Closed(void)		//Validate that Entrance servo 2 is Closed 
{
Validate_Entrance_Opening(2, 0x10,  0x1000);	//check servo 2 position
}

void EntranceServo3_Open(void)		//Validate that Entrance servo 3 is Open 
{
Validate_Entrance_Opening(3, 0x10,  0x2000);	//check servo 3 position
}

void EntranceServo3_Closed(void)		//Validate that Entrance servo 3 is Closed 
{
Validate_Entrance_Opening(3, 0x10,  0x1000);	//check servo 3 position
}

void EntranceServo4_Open(void)		//Validate that Entrance servo 4 is Open 
{
Validate_Entrance_Opening(4, 0x10,  0x2000);	//check servo 4 position
}

void EntranceServo4_Closed(void)		//Validate that Entrance servo 4 is Closed 
{
Validate_Entrance_Opening(4, 0x10,  0x1000);	//check servo 4 position
}

void EntranceServo5_Open(void)		//Validate that Entrance servo 5 is Open 
{
Validate_Entrance_Opening(5, 0x10,  0x2000);	//check servo 5 position
}

void EntranceServo5_Closed(void)		//Validate that Entrance servo 5 is Closed 
{
Validate_Entrance_Opening(5, 0x10,  0x1000);	//check servo 5 position
}

void EntranceServo6_Open(void)		//Validate that Entrance servo 6 is Open 
{
Validate_Entrance_Opening(6, 0x10,  0x2000);	//check servo 6 position
}

void EntranceServo6_Closed(void)		//Validate that Entrance servo 6 is Closed 
{
Validate_Entrance_Opening(6, 0x10,  0x1000);	//check servo 6 position
}

void EntranceServo7_Open(void)		//Validate that Entrance servo 7 is Open 
{
Validate_Entrance_Opening(7, 0x10,  0x2000);	//check servo 7 position
}

void EntranceServo7_Closed(void)		//Validate that Entrance servo 7 is Closed 
{
Validate_Entrance_Opening(7, 0x10,  0x1000);	//check servo 7 position
}

void EntranceServo8_Open(void)		//Validate that Entrance servo 8 is Open 
{
Validate_Entrance_Opening(8, 0x10,  0x2000);	//check servo 8 position
}

void EntranceServo8_Closed(void)		//Validate that Entrance servo 8 is Closed 
{
Validate_Entrance_Opening(8, 0x10,  0x1000);	//check servo 8 position
}

