
//=================================================
//
//	BMS_Global.h
//
//=================================================
#ifndef __BMS_GLOBAL_H
#define __BMS_GLOBAL_H


// BMS Global variables, used in BeeTrafficCounter.c
// WORD size of 16 bits will handle a maximum of 18 bees per second average per hour
	WORD BeeOutCounter;			//keeps track of total number of bee leaving hive, each hour it is reset
	WORD BeeInCounter;			//keeps track of total number of bees entering hive, each hour it is reset

	

// BMS Global variables, used in BMS_Entrance_Servo.c 
//These valuesd are used to program the PWM for each servo's position
unsigned short int	Previous_Servo_Start[8];
unsigned short int	Previous_Servo_Stop[8];
DWORD	tgServo;		//temp test of tick

BYTE ServoBusyFlag;		//TRUE Indicates a servo is still being positioned, new position cmd can not be accepted yet

//===============================================
//
//	Sensor & Function defines for memory array indexing
//
//==============================================
//
//	Hive configuration
//		Sensor hardware- item name
//		Present-yes/no
//		Enabled-yes/no
//
// 		Hardware		Function
// 		or function	 	number				Description
//
// Baseline functions
#define	E12vmon				1			//+12v power supply monitor
#define	E5vmon				2			//+5v power supply monitor
#define	E3p3vmonP32			3			//+3.3v P32_VDD supply monitor
#define	E3p3vmonIO			4			//+3.3v IO_PWR supply monitor
#define	E3p3vmonFRAME		5			//+3.3v FRAME_PWR supply monitor
#define	EsysconTEMP			6			//system controller MCP9801 temperature monitor, addr=0x90
#define	EbaseinTEMP			7			//hive base inside MCP9801 temperature monitor, addr=0x92
#define	EtopinTEMP			8			//hive top inside MCP9801 temperature monitor, addr=0x94
#define	EbaseoutTEMP		9			//hive base outside MCP9801 temperature monitor, addr=0x96
#define	EtopoutTEMP			10			//hive top inside MCP9801 temperature monitor, addr=0x98
#define	Espare1TEMP			11			//spare1 MCP9801 temperature monitor, addr=0x9A
#define	Espare2TEMP			12			//spare2 MCP9801 temperature monitor, addr=0x9C
#define	Espare3TEMP			13			//spare3 MCP9801 temperature monitor, addr=0x9E
#define	ERH					14			//hive humidity sensor, Honeywell HIH-4031, software displays %RH
#define	Edaylight			15			//daylight LDR sensor, software displays a range of 0-100%
#define EaudioIn			16			//microphone for hive audio analysis
//
// Security functions
#define	Elidopen			17			//LOS, Si1_Si2,  hive lid open hall effect sensor, SS411A, 1=open, 0=closed
#define	Etiltdetect			18			//TD hive tilt detect hall effect sensor, SS411A, 1=hive tilted, 0=level
#define EPIR1ms				19			//PIR1 motion sensor
#define EPIR2ms				20			//PIR2 motion sensor
#define EPIR3ms				21			//PIR3 motion sensor
#define EPIR4ms				22			//PIR4 motion sensor
#define Eelecfencemode		23			//EFC electric fence configuration mode, 3 state
#define	EelectfenceLED		24			//EFPled electric fence LED controller power opto output
#define	EtamperalarmLED		25			//TAled Tamper Alarm LED opto output
#define	EtiltalarmLED		26			//HTled Hive Tilt Alarm LED opto output
#define	EPIRmotionalarmLED	27			//PIRled Motion Alarm LED opto output
//
// General functions
#define EBeeOutLED			28			//BLHled Bee Leaving hive LED opto output
#define EBeeInLED			29			//BEHled Bee Entering hive LED opto output
#define	ESmokeLED			30			//SMled Smoke module LED opto output
#define ERainLED			31			//RMled Rain module LED opto output
#define	ERTCCalarmLED		32			//RTCCled RTCC alarm LED opto output
#define	ESoundPWMLED		33			//Spwmled Sound PWM LED opto output
#define	EVibrationLED		34			//Vpwmled Vibration PWM LED opto output
#define	EEntrancePWMLED		35			//Epwnled Entrance opening PWM LED opto output
#define	EHeaterPWMLED		36			//Hpwmled Heater PWM LED opto output
#define	ECoolingPWMLED		37			//Cpwmled Cooling PWM LED opto output
#define EHiveStatGrnLED		38			//HSGrnled Hive status green LED opto output
#define EHiveStatYelLED		39			//HSYelled Hive status yellow LED opto output
#define EHiveStatRedLED		40			//HSRedled Hive status red LED opto output
#define EHiveExteriorLED	41			//HELled Hive exterior light LED opto output
#define EHiveInteriorLED	42			//HILled Hive interior light LED opto output
#define EGPSModule			43			//GPS Hive GPS module interface, UART1
#define EEnviroLink			44			//EL External Ambient Enviromental Link, UART2 (diagnostic use also)
//
// Bee traffic tunnels
#define	ETrafficCounter1	45			//Bee traffic counter 1
#define	ETrafficCounter2	46			//Bee traffic counter 2
#define	ETrafficCounter3	47			//Bee traffic counter 3
#define	ETrafficCounter4	48			//Bee traffic counter 4
#define	ETrafficCounter5	49			//Bee traffic counter 5
#define	ETrafficCounter6	50			//Bee traffic counter 6
#define	ETrafficCounter7	51			//Bee traffic counter 7
#define	ETrafficCounter8	52			//Bee traffic counter 8
#define	EIRbeeTemp1			53			//Bee non-contact IR temperature sensor 1
#define	EIRbeeTemp2			54			//Bee non-contact IR temperature sensor 2
#define	EIRbeeTemp3			55			//Bee non-contact IR temperature sensor 3
#define	EIRbeeTemp4			56			//Bee non-contact IR temperature sensor 4
#define	EIRbeeTemp5			57			//Bee non-contact IR temperature sensor 5
#define	EIRbeeTemp6			58			//Bee non-contact IR temperature sensor 6
#define	EIRbeeTemp7			59			//Bee non-contact IR temperature sensor 7
#define	EIRbeeTemp8			60			//Bee non-contact IR temperature sensor 8
#define	EbeeScale1			61			//Bee scale module 1
#define	EbeeScale2			62			//Bee scale module 2
#define	EbeeScale3			63			//Bee scale module 3
#define	EbeeScale4			64			//Bee scale module 4
#define	EbeeScale5			65			//Bee scale module 5
#define	EbeeScale6			66			//Bee scale module 6
#define	EbeeScale7			67			//Bee scale module 7
#define	EbeeScale8			68			//Bee scale module 8
//
// Servo power enables, saves wear and tear on the servo motors
#define ServoPwr1			69			//Servo 1 power enable/disable
#define ServoPwr2			70			//Servo 2 power enable/disable
#define ServoPwr3			71			//Servo 3 power enable/disable
#define ServoPwr4			72			//Servo 4 power enable/disable
#define ServoPwr5			73			//Servo 5 power enable/disable
#define ServoPwr6			74			//Servo 6 power enable/disable
#define ServoPwr7			75			//Servo 7 power enable/disable
#define ServoPwr8			76			//Servo 8 power enable/disable
//
// Auxiliary Devices
#define AuxDev1				77			//Auxiliary device 1
#define AuxDev2				78			//Auxiliary device 2
#define AuxDev3				79			//Auxiliary device 3
#define AuxDev4				80			//Auxiliary device 4
#define AuxDev5				81			//Auxiliary device 5
#define AuxDev6				82			//Auxiliary device 6
#define AuxDev7				83			//Auxiliary device 7
#define AuxDev8				84			//Auxiliary device 8
#define DataLogFlag			85			//Data Logging Flag
#define AlarmWaitTimeReset	86			//Alarm Wait Time Reset function
#define EmailAlertEnableFlag 87			//Enables or Disables emailing of alerts and status
#define EmailDLogEnableFlag  88			//Enables or Disables emailing of data logging
#define ErtccCalValue		89			//RTCC calibration value stored in DataValue1 as signed DWORD
// NOTE: If adding more items, increase the RAM array size "FunctionModuleSize" below.
#define FunctionModuleSize	(89+1)		//since index 0 is skipped,
										// make sure there is enough memory +1 from above #defines
typedef struct
{
	BYTE	Installed:1;
	BYTE	Enabled:1;
	BYTE	AlarmEnable:1;
	BYTE	AlarmLower:1;
	BYTE	AlarmUpper:1;
} ESTATUS_BITS;

typedef  struct 
{
	ESTATUS_BITS Status;		//see above struct ESTATUS_BITS for status flags			
	WORD Lowerlimit;			//Lower limit
	WORD Upperlimit;			//Upper limit
	WORD DataValue1;			//Data value 1 as needed
	WORD DataValue2;			//Data value 2 as needed
} FUNCTION_RECORD;


// The wait time flags below are used to hold off from sending an overload of alarm emails for the same
// condition repeatedly.
typedef  struct 
{
	WORD	LidOpenFlag:1;		//Lid Open wait time flag
	WORD	TiltFlag:1;			//Tilt sensor wait time flag
	WORD	PIR1Flag:1;			//PIR 1 wait time flag
	WORD	PIR2Flag:1;			//PIR 2 wait time flag
	WORD	PIR3Flag:1;			//PIR 3 wait time flag
	WORD	PIR4Flag:1;			//PIR 4 wait time flag
	WORD	Vm12Flag:1;			//+12 volt supply monitor wait time flag
	WORD	Vm5Flag:1;			//+5 volt supply monitor wait time flag
	WORD	Vm3P32Flag:1;		//+3 volt P32_VDD supply monitor wait time flag
	WORD	Vm3IOFlag:1;		//+3 volt IO_PWR supply monitor wait time flag
	WORD	Vm3FrameFlag:1;		//+3 volt FRAME_PWR supply monitor wait time flag
	WORD	TempSysConFlag:1;	//Temperature system controller wait time flag
} ALARM_FLAGS_RECORD;
 

// In Main.c
//RAM array that holds current sensor and function status and values
//FUNCTION_RECORD FunctionModule[FunctionModuleSize];		//since index 0 is skipped,
										// make sure there is enough memory +1 from above #defines
// useage:
BYTE tempvalue;
//FunctionModule[E12vmon].Status.Installed = TRUE;		//TRUE is defined as 0x01, FALSE as 0x00
//FunctionModule[E12vmon].Status.Enabled = TRUE;
//FunctionModule[E12vmon].Status.AlarmEnable = TRUE;
//FunctionModule[E12vmon].Status.AlarmLower = TRUE;
//FunctionModule[E12vmon].Status.AlarmUpper = FALSE;
//FunctionModule[E12vmon].Upperlimit = 0x0F05;
//tempvalue = 0x0F;
//tempvalue = FunctionModule[E12vmon].Status.Enabled;
//tempvalue = FunctionModule[E12vmon].Status.Installed;
//tempvalue = FunctionModule[E12vmon].Lowerlimit;
//tempvalue = 0;


// The following are now enum in CustomHTTPApp.c around line 119, as FUNCTION_TO_GRAPH
// 		Graph				Graph
// 	 	name  				number		Graph Description
//#define gTSysController	0			//System controller temperature
//#define gTBaseOutSide		1			//Hive Base temperature outside
//#define gTBaseInside		2			//Hive Base temperature inside
//#define gTTopOutside		3			//Hive Top temperature outside
//#define gTTopInside		4			//Hive Top temperature inside
//#define gHumidity			5			//Hive humidity
//#define gDayLight			6			//Day Light Sensor
//#define gBeeTrafficOut	7			//Bee Traffic Out
//#define gBeeTrafficIn		8			//Bee Traffic In
// NOTE: If adding more items, increase the RAM array size "NumberGraphs" below.
#define NumberGraphs		(8+1)		//make sure there is enough memory +1 from above #defines of graph names


typedef  struct 
{
	WORD cTSysController;		//System Controller temperature
	WORD cTBaseOutSide;		//Hive Base temperature outside
	WORD cTBaseInside;			//Hive Base temperature inside
	WORD cTTopOutside;			//Hive Top temperature outside
	WORD cTTopInside;			//Hive Top temperature inside
	WORD cHumidity;				//Hive humidity
	WORD cDayLight;				//Day Light Sensor			
	WORD cBeeTrafficOut;		//Bee Traffic Out
	WORD cBeeTrafficIn;			//Bee Traffic In
} SENSORLOG_RECORD;

#define NumberLogDays		30		//Number of days to log in EEPROM
#define DataLogInterval		5		//seconds after dataloging to clear flags
			

static enum				//datalog reading names
{
	Reading_1 = 0,
	Reading_2,
	Reading_3,
	Reading_4
} DATALOGREADING = Reading_1;

static enum		//states for data sets to output
{
	DATA_ID_HOME = 0,		
	DATA_ID_BEEOUT,			//Bee Traffic Out
	DATA_ID_BEEIN,			//Bee Traffic In
	DATA_ID_DAYLIGHT,		//Daylight sensor
	DATA_ID_TBO,			//Temperature Base Outside
	DATA_ID_TBI,			//Temperature Base Inside
	DATA_ID_TTO,			//Temperature Top Outside
	DATA_ID_TTI,			//Temperature Top Inside
	DATA_ID_HH,				//Hive Humidity
	DATA_ID_DONE
} DataID = DATA_ID_HOME;

// ADC scale factors
#define adc_12vmon_scale_factor 537

#endif

