//============================================
//	BMS Hardware Configuration
//============================================
#ifndef __BMS_HARDWARECONFIG_H
#define __BMS_HARDWARECONFIG_H
// Define your own board hardware profile here
//actually a copy of the Explorer 16 code, 
// Seems like there is a bug that screws up the UART output if we define YOUR_BOARD
// Instead make this a copy of the Explorer 16 configuration and modify the Explorer 16
// code to be our BMS code!

// Explorer 16 + PIC24FJ128GA010/PIC24HJ256GP610/dsPIC33FJ256GP710 PIM + Ethernet PICtail Plus
// Explorer 16 + PIC32MX460F512L/PIC32MX360F512L PIM + Ethernet PICtail Plus

//defines used in MCP9800_lib.c, repeated here for reference only
#define SYSCLK	(80000000ul)
#define PBCLK  (SYSCLK/2)
#define Fsck	50000
#define BRG_VAL 	((PBCLK/2/Fsck)-2)


// BMS LED defs for the project
	#define GreenStatusLEDtris	(TRISDbits.TRISD5)  // Green Status LED
	#define GreenStatusLEDio	(LATDbits.LATD5)
	#define YellowStatusLEDtris	(TRISGbits.TRISG1)  // Yellow Status LED
	#define YellowStatusLEDio	(LATGbits.LATG1)
	#define RedStatusLEDtris	(TRISGbits.TRISG0)  // Red Status LED
	#define RedStatusLEDio		(LATGbits.LATG0)

// General Note: OPTO-ISOLATOR outputs are available for all LED indications
// Tamper Alarm LED output is set by either the Lid Open Sensor or the Hive Movement Sensor being triped
// This Alarm output does not auto reset, use SW2 to reset tripped conditions or WEB page
	#define Tamper_Alarm_LEDtris			(TRISEbits.TRISE4)
 	#define Tamper_Alarm_LEDio				(LATEbits.LATE4)

// Hive Movement Alarm is set by the Tilt Hall Sensor being tripped
// This Alarm output does not auto reset, use SW2 to reset tripped conditions or WEB page
	#define Hive_Movement_Alarm_LEDtris		(TRISEbits.TRISE3)
 	#define Hive_Movement_Alarm_LEDio		(LATEbits.LATE3)

// PIR Motion Alarm is set by any of the PIR1, PIR 2, PIR 3, or PIR 4 detectors being triggered
// This alarm output automatically resets after set timeout configuredon WEB page
	#define PIR_Motion_Alarm_LEDtris		(TRISEbits.TRISE2)
 	#define PIR_Motion_Alarm_LEDio			(LATEbits.LATE2)

// Bee leaving hive LED is toggled by the bee traffic coounter detecting a bee is leaving the hive
// Each bee counter will toggle this LED, a total count of bees for each hour in the last 24 hours
// is logged.  Additional days may be kept in log.
	#define Bee_Leaving_Hive_LEDtris		(TRISDbits.TRISD13)
 	#define Bee_Leaving_Hive_LEDio			(LATDbits.LATD13)

// Bee entering hive LED is toggled by the bee traffic coounter detecting a bee is entering the hive
// Each bee counter will toggle this LED, a total count of bees for each hour in the last 24 hours
// is logged.  Additional days may be kept in log.
	#define Bee_Entering_Hive_LEDtris		(TRISDbits.TRISD12)
 	#define Bee_Entering_Hive_LEDio			(LATDbits.LATD12)	
	
// Smoke module drive LED is enabled/set by WEB page 
	#define Smoke_Module_LEDtris			(TRISDbits.TRISD9)
 	#define Smoke_Module_LEDio				(LATDbits.LATD9)

// BTCSW drive is enabled
// BTCSW will use GPIOB bit 1 for Bee Traffic LED power on/off, this is used for diagnostics when it needs to be turned off
	#define BTCSW_tris				(0x01)	//Read IODIRB port first and then use this value to set bit as output
 	#define BTCSW_io				(0x01)	//bit to set or clear to control output

// Rain module drive LED is enabled/set by WEB page 
// Rain_Module will now use GPIOB bit 2 for AUX-3 IO
	#define Rain_Module_LEDtris				(0x04)	//Read IODIRB port first and then use this value to set bit as output
 	#define Rain_Module_LEDio				(0x04)	//bit to set or clear to control output

// AuxDev 4 input is enabled for reading a jumper to determine if we are to set the TestModeBMS TRUE 
// uses MCP23017 Addr=48, GPB3
	#define AuxDev4_tris					(0x08)	//Read IODIRB port first and then use this value to set bit as input
	#define AuxDev4_io						(0x08)	//bit to read

// AuxDev 5 drive LED is enabled/set by WEB page 
// uses MCP23017 Addr=48, GPB4
	#define AuxDev5_LEDtris					(0x10)	//Read IODIRB port first and then use this value to set bit as output
	#define AuxDev5_LEDio					(0x10)	//bit to set or clear to control output

// AuxDev 6 drive LED is enabled/set by WEB page 
// uses MCP23017 Addr=48, GPB4
	#define AuxDev6_LEDtris					(0x20)	//Read IODIRB port first and then use this value to set bit as output
	#define AuxDev6_LEDio					(0x20)	//bit to set or clear to control output

// AuxDev 7 drive LED is enabled/set by WEB page 
// uses MCP23017 Addr=48, GPB4
	#define AuxDev7_LEDtris					(0x40)	//Read IODIRB port first and then use this value to set bit as output
	#define AuxDev7_LEDio					(0x40)	//bit to set or clear to control output

// AuxDev 8 drive LED is enabled/set by WEB page 
// uses MCP23017 Addr=48, GPB4
	#define AuxDev8_LEDtris					(0x80)	//Read IODIRB port first and then use this value to set bit as output
	#define AuxDev8_LEDio					(0x80)	//bit to set or clear to control output

// Entrance module servo power LED is enabled/set by WEB page 
// This signal provide a methode to turn power on and off to the servo motor used at the entrance
// this conserves both power and lengthens the life of the servo motors
// ######### Update these defines, RD11 is now used for I2C1 Data Memory write protect enable WP
// MCP23017 at address=48(7bit), GPA0=Servo#1.....GPA7=Servo#8
	#define ServoPower1_LEDtris			(0x01)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower1_LEDio			(0x01)	//bit to set or clear to control output
	#define ServoPower2_LEDtris			(0x02)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower2_LEDio			(0x02)	//bit to set or clear to control output
	#define ServoPower3_LEDtris			(0x04)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower3_LEDio			(0x04)	//bit to set or clear to control output
	#define ServoPower4_LEDtris			(0x08)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower4_LEDio			(0x08)	//bit to set or clear to control output
	#define ServoPower5_LEDtris			(0x10)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower5_LEDio			(0x10)	//bit to set or clear to control output
	#define ServoPower6_LEDtris			(0x20)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower6_LEDio			(0x20)	//bit to set or clear to control output
	#define ServoPower7_LEDtris			(0x40)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower7_LEDio			(0x40)	//bit to set or clear to control output
	#define ServoPower8_LEDtris			(0x80)	//Read IODIRA port first and then use this value to set bit as output
 	#define ServoPower8_LEDio			(0x80)	//bit to set or clear to control output

// Electric fence module drive LED is enabled/set by WEB page 
	#define Fence_Module_LEDtris			(TRISEbits.TRISE0)
 	#define Fence_Module_LEDio				(LATEbits.LATE0)

// RTCC alarm clock module drive LED is enabled/set by WEB page 

// Sound module PWM drive OC1

// Vibration module PWM drive

// Entrance module PWM servo drive

// Heater module PWM drive or digital on/off drive
	#define HeatingModule_LEDtris			(TRISDbits.TRISD3)
	#define HeatingModule_LEDio				(LATDbits.LATD3)

// Cooling module PWM or digital on/off drive
	#define CoolingModule_LEDtris			(TRISDbits.TRISD4)
	#define CoolingModule_LEDio				(LATDbits.LATD4)

// Hive exterior light module
	#define ExteriorLight_Module_LEDtris	(TRISFbits.TRISF0)
 	#define ExteriorLight_Module_LEDio		(LATFbits.LATF0)

// Hive interior light module
	#define InteriorLight_Module_LEDtris	(TRISFbits.TRISF1)
 	#define InteriorLight_Module_LEDio		(LATFbits.LATF1)

// MCP23017's
// /RESET is done by RD10
	#define MCP23017_Reset_tris				(TRISDbits.TRISD10)
	#define MCP23017_Reset_io				(LATDbits.LATD10)

// U910 AUX lines, Slave addr = 48
//	#define Rain_Module_LEDtris				(TRISDbits.TRISD10)
//	#define Rain_Module_LEDio				(LATDbits.LATD10)

// I2C modules
//
// I2C1 Data Memory write protect enable WP on I2C1
// RD11 is now used for Data Log Memory Protect
	#define Data_1_Memory_tris		(TRISDbits.TRISD11)
	#define Data_1_Memory_io		(LATDbits.LATD11)

// MCP9801's for general hive temperature sensing
	#define	SysConTempPort	0x01		//I2C port# 	system controller temperature
	#define	SysConTempSA	0x00		//SlaveAddress

	#define	BaseInTempPort	0x01		//I2C port# 	hive base inside temperature
	#define	BaseInTempSA	0x01		//SlaveAddress

	#define	TopInTempPort	0x01		//I2C port# 	hive top inside temperature
	#define	TopInTempSA		0x02		//SlaveAddress

	#define	BaseOutTempPort	0x01		//I2C port# 	hive base outside temperature
	#define	BaseOutTempSA	0x03		//SlaveAddress

	#define	TopOutTempPort	0x01		//I2C port# 	hive top outside temperature
	#define	TopOutTempSA	0x04		//SlaveAddress

// Bee Traffic Counters use MCP23017
	#define BeeTrafficPort	0x02		//I2C port# 	bee traffic counters
	#define BeeTrafficSA  	0x42		//Device and SlaveAddress, 7-bit

// Aux io use MCP23017
	#define AuxioPort		0x02		//I2C port# 	bee traffic counters
	#define AuxioSA  		0x48		//Device and SlaveAddress, 7-bit

//MCP23017 registers
	#define IODIRA			0x00		//BANK = 1 
	#define GPIOA			0x09
	#define OLATA			0x0A
	#define IODIRB			0x10
	#define GPIOB			0x19
	#define OLATB			0x1A


//	#define LED_GET()			(*((volatile unsigned char*)(&LATA)))
//	#define LED_PUT(a)			(*((volatile unsigned char*)(&LATA)) = (a))



// BMS switch defs for the project
	#define BUTTON_SW1_TRIS		(TRISDbits.TRISD6)	//PB switch for diagnostics
	#define	BUTTON_SW1_IO		(PORTDbits.RD6)
	#define BUTTON_SW2_TRIS		(TRISDbits.TRISD7)	//PB switch for diagnostics
	#define	BUTTON_SW2_IO		(PORTDbits.RD7)

	#define SW_LidOpen_Tris		(TRISEbits.TRISE7)
	#define SW_LidOpen_IO		(PORTEbits.RE7)
	#define SW_TiltDetect_Tris	(TRISEbits.TRISE6)
	#define SW_TiltDetect_IO	(PORTEbits.RE6)

//PIR Modules
	#define SW_PIR_1_TRIS		(TRISEbits.TRISE5)
	#define SW_PIR_1_IO			(PORTEbits.RE5)

	#define SW_PIR_2_TRIS		(TRISGbits.TRISG9)
	#define SW_PIR_2_IO			(PORTGbits.RG9)

	#define SW_PIR_3_TRIS		(TRISAbits.TRISA9)
	#define SW_PIR_3_IO			(PORTAbits.RA9)

	#define SW_PIR_4_TRIS		(TRISAbits.TRISA10)
	#define SW_PIR_4_IO			(PORTAbits.RA10)
			

// 	#define BUTTON0_TRIS		(TRISDbits.TRISD13)	// Ref S4
//	#define	BUTTON0_IO			(PORTDbits.RD13)
//	#define BUTTON1_TRIS		(TRISAbits.TRISA7)	// Ref S5	// Note: This is multiplexed with LED7
//	#define	BUTTON1_IO			(PORTAbits.RA7)
//	#define BUTTON2_TRIS		(TRISDbits.TRISD7)	// Ref S6
//	#define	BUTTON2_IO			(PORTDbits.RD7)
//	#define BUTTON3_TRIS		(TRISDbits.TRISD6)	// Ref S3
//	#define	BUTTON3_IO			(PORTDbits.RD6)

//	#define BUTTON2_TRIS		(TRISDbits.TRISD6)	// Ref S6
//	#define	BUTTON2_IO			(PORTDbits.RD6)
//	#define BUTTON3_TRIS		(TRISDbits.TRISD13)	// Ref S3
//	#define	BUTTON3_IO			(PORTDbits.RD13)

	#define UARTTX_TRIS			(TRISFbits.TRISF5)
	#define UARTTX_IO			(PORTFbits.RF5)
	#define UARTRX_TRIS			(TRISFbits.TRISF4)
	#define UARTRX_IO			(PORTFbits.RF4)

	// ENC28J60 I/O pins
	#define ENC_RST_TRIS		(TRISDbits.TRISD15)	// Not connected by default
	#define ENC_RST_IO			(PORTDbits.RD15)
	#define ENC_CS_TRIS			(TRISDbits.TRISD14)
	#define ENC_CS_IO			(PORTDbits.RD14)
	// SPI SCK, SDI, SDO pins are automatically controlled by the 
	// PIC24/dsPIC/PIC32 SPI module 
	#if defined(__C30__)	// PIC24F, PIC24H, dsPIC30, dsPIC33
		#define ENC_SPI_IF			(IFS0bits.SPI1IF)
		#define ENC_SSPBUF			(SPI1BUF)
		#define ENC_SPISTAT			(SPI1STAT)
		#define ENC_SPISTATbits		(SPI1STATbits)
		#define ENC_SPICON1			(SPI1CON1)
		#define ENC_SPICON1bits		(SPI1CON1bits)
		#define ENC_SPICON2			(SPI1CON2)
	#else					// PIC32
		#define ENC_SPI_IF			(IFS0bits.SPI1RXIF)
		#define ENC_SSPBUF			(SPI1BUF)
		#define ENC_SPICON1			(SPI1CON)
		#define ENC_SPICON1bits		(SPI1CONbits)
		#define ENC_SPIBRG			(SPI1BRG)
	#endif

	// 25LC256 I/O pins
// 	#define EEPROM_CS_TRIS		(TRISDbits.TRISD12)		// orig configuration
// 	#define EEPROM_CS_IO		(PORTDbits.RD12)		// orig configuration

	#define EEPROM_CS_TRIS		(TRISCbits.TRISC1)		//BMS configuration
	#define EEPROM_CS_IO		(PORTCbits.RC1)		//BMS configuration

	#define EEPROM_SCK_TRIS		(TRISGbits.TRISG6)
	#define EEPROM_SDI_TRIS		(TRISGbits.TRISG7)
	#define EEPROM_SDO_TRIS		(TRISGbits.TRISG8)
	#if defined(__C30__)	// PIC24F, PIC24H, dsPIC30, dsPIC33
		#define EEPROM_SPI_IF		(IFS2bits.SPI2IF)
		#define EEPROM_SSPBUF		(SPI2BUF)
		#define EEPROM_SPICON1		(SPI2CON1)
		#define EEPROM_SPICON1bits	(SPI2CON1bits)
		#define EEPROM_SPICON2		(SPI2CON2)
		#define EEPROM_SPISTAT		(SPI2STAT)
		#define EEPROM_SPISTATbits	(SPI2STATbits)
	#else					// PIC32
		#define EEPROM_SPI_IF		(IFS1bits.SPI2RXIF)
		#define EEPROM_SSPBUF		(SPI2BUF)
		#define EEPROM_SPICON1		(SPI2CON)
		#define EEPROM_SPICON1bits	(SPI2CONbits)
		#define EEPROM_SPIBRG		(SPI2BRG)
	#endif

	// LCD Module I/O pins
//	#define LCD_DATA_TRIS		(*((volatile BYTE*)&TRISE))
//	#define LCD_DATA_IO			(*((volatile BYTE*)&LATE))
//	#define LCD_RD_WR_TRIS		(TRISDbits.TRISD5)
//	#define LCD_RD_WR_IO		(LATDbits.LATD5)
//	#define LCD_RS_TRIS			(TRISBbits.TRISB15)
//	#define LCD_RS_IO			(LATBbits.LATB15)
//	#define LCD_E_TRIS			(TRISDbits.TRISD4)
//	#define LCD_E_IO			(LATDbits.LATD4)

//	// Serial Flash/SRAM/UART PICtail Plus attached to SPI2 (middle pin group)
//	#define SPIRAM_CS_TRIS			(TRISGbits.TRISG9)
//	#define SPIRAM_CS_IO			(LATGbits.LATG9)
//	#define SPIRAM_SCK_TRIS			(TRISGbits.TRISG6)
//	#define SPIRAM_SDI_TRIS			(TRISGbits.TRISG7)
//	#define SPIRAM_SDO_TRIS			(TRISGbits.TRISG8)
//	#if defined(__C30__)	// PIC24F, PIC24H, dsPIC30, dsPIC33
//		#define SPIRAM_SPI_IF			(IFS2bits.SPI2IF)
//		#define SPIRAM_SSPBUF			(SPI2BUF)
//		#define SPIRAM_SPICON1			(SPI2CON1)
//		#define SPIRAM_SPICON1bits		(SPI2CON1bits)
//		#define SPIRAM_SPICON2			(SPI2CON2)
//		#define SPIRAM_SPISTAT			(SPI2STAT)
//		#define SPIRAM_SPISTATbits		(SPI2STATbits)
//	#else					// PIC32
//		#define SPIRAM_SPI_IF			(IFS1bits.SPI2RXIF)
//		#define SPIRAM_SSPBUF			(SPI2BUF)
//		#define SPIRAM_SPICON1			(SPI2CON)
//		#define SPIRAM_SPICON1bits		(SPI2CONbits)
//		#define SPIRAM_SPIBRG			(SPI2BRG)
//	#endif
//	#define SPIFLASH_CS_TRIS		(TRISBbits.TRISB8)
//	#define SPIFLASH_CS_IO			(LATBbits.LATB8)
//	#define SPIFLASH_SCK_TRIS		(TRISGbits.TRISG6)
//	#define SPIFLASH_SDI_TRIS		(TRISGbits.TRISG7)
//	#define SPIFLASH_SDI_IO			(PORTGbits.RG7)
//	#define SPIFLASH_SDO_TRIS		(TRISGbits.TRISG8)
//	#if defined(__C30__)	// PIC24F, PIC24H, dsPIC30, dsPIC33
//		#define SPIFLASH_SPI_IF			(IFS2bits.SPI2IF)
//		#define SPIFLASH_SSPBUF			(SPI2BUF)
//		#define SPIFLASH_SPICON1		(SPI2CON1)
//		#define SPIFLASH_SPICON1bits	(SPI2CON1bits)
//		#define SPIFLASH_SPICON2		(SPI2CON2)
//		#define SPIFLASH_SPISTAT		(SPI2STAT)
//		#define SPIFLASH_SPISTATbits	(SPI2STATbits)
//	#else					// PIC32
//		#define SPIFLASH_SPI_IF			(IFS1bits.SPI2RXIF)
//		#define SPIFLASH_SSPBUF			(SPI2BUF)
//		#define SPIFLASH_SPICON1		(SPI2CON)
//		#define SPIFLASH_SPICON1bits	(SPI2CONbits)
//		#define SPIFLASH_SPIBRG			(SPI2BRG)
//	#endif

#endif

