//==========================================================
//
//	BeeTrafficCount.c
//	Bee Traffic Counters
//
//==========================================================
#include <plib.h>
#include "GenericTypeDefs.h"
#include "BMS_Globals.h"
//#include "MCP23017_lib.c"		// included in Source files BMS Library folder

// Configuration Bit settings
// SYSCLK = 80 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 40 MHz
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
//
//#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
//#pragma config POSCMOD = HS, FNOSC = PRIPLL, FPBDIV = DIV_2


#define SYSCLK	(80000000ul)
#define PBCLK  (SYSCLK/2)
#define Fsck	50000
#define BRG_VAL 	((PBCLK/2/Fsck)-2)

//==============================================================
//  
//	Bee traffic Counter using
// 	MCP23017 16-Bit I2C IO Expander on I2C2 port
//
//	If Trigger =1 and Direction =0 = Bee entering hive
//	If Trigger =1 and Direction =1 = Bee leaving hive
//
//==============================================================

static unsigned char TrafficPort;		//I2C port number
static BYTE TrafficSA = 0x42;			//I2C slave address
//static WORD EnabledBeeTrafficCounters = 0x0300;	//Counter 1,Installed and enabled bee traffic counters, both triggers and direction bits
//static WORD EnabledBeeTrafficCounters = 0xFF00;	//Counters 1-4,Installed and enabled bee traffic counters, both triggers and direction bits
//static WORD EnabledBeeTrafficCounters = 0xFFFF;	//Counters 1-8,Installed and enabled bee traffic counters, both triggers and direction bits
static WORD EnabledBeeTrafficCounters;	//Counters 1-8 configured from web page

// Prototypes
char MCP23017_readRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg);
void MCP23017_writeRegister(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char Reg, unsigned char Data);
void MCP23017_setBANK_1(unsigned char I2Cport, unsigned char SlaveAddr, unsigned char ConfigData);

void InitializeBeeTrafficCounters(unsigned char Port)
{
// Global variables
	extern BeeInCounter;			//keeps track of total number of bees entering hive
	extern BeeOutCounter;			//keeps track of total number of bee leaving hive

static WORD BeeTrigger = 0;
static WORD BeeDirection = 0;
static WORD BeeClear = 0;

	if (Port == 1)
	{
	TrafficPort = 1;
	NotAckI2C1();
	CloseI2C1();
	OpenI2C1( I2C_EN, BRG_VAL );		//Enable I2C1 port
	}
		else
		{
		TrafficPort = 2;
		NotAckI2C2();
		CloseI2C2();
		OpenI2C2( I2C_EN, BRG_VAL );	//Enable I2C2 port
		}

// Configure the Bee Traffic Counter MCP23017 IO Expander registers
//
// Configure ICON register first, because other registers map differently otherwise
	MCP23017_setBANK_1(TrafficPort, TrafficSA, 0xEC);		// MCP23017 16-BIT IO Expander
	MCP23017_setBANK_1(0x02, 0x48, 0xEC);		// MCP23017 16-BIT IO Expander


// Configure MCP23017 for all inputs
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x00, 0xFF);	//set IODIRA register all inputs
// Need to configure all MCP23017's using the same INT4 net to the PIC32 as Open-Drain,
// otherwise they hold the line down
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x10, 0xFF);	//set IODIRB register all inputs

// Configure input polarity
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x01, 0x00);	//set IPOLA register non-invert
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x11, 0x00);	//set IPOLB register non-invert

// Configure for interrupt-on-change, bottom IR triggers interrupt, top IR is direction
// Wired to INT4 on PIC32
// Interrupts off
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x02, 0x00);	//set GPINTENA register non-invert
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x12, 0x00);	//set GPINTENB register non-invert, 0x02 int

// Configure default compare register
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x03, 0x00);	//set DEFVALA register non-invert
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x13, 0x00);	//set DEFVALB register non-invert

// Configure interrupt control register
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x04, 0xFF);	//set INTCONA register 1=use DEFVALA
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x14, 0xFF);	//set INTCONB register 1=use DEFVALB

// Configure for internal pull-ups on port, may be omitted because resistor is on the board
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x06, 0xFF);	//set GPPUA register pull-ups
	MCP23017_writeRegister(TrafficPort, TrafficSA, 0x16, 0xFF);	//set GPPUB register pull-ups

}


void CheckBeeCounters(void)
{
// Global variables
//	extern BeeInCounter;			//keeps track of total number of bees entering hive
//	extern BeeOutCounter;			//keeps track of total number of bee leaving hive

WORD BeeData;
WORD BeeTrigger_01, BeeDirection_01;	//trigger and direction indicators for tunnel 1
WORD BeeTrigger_02, BeeDirection_02;	//trigger and direction indicators for tunnel 2
WORD BeeTrigger_03, BeeDirection_03;	//trigger and direction indicators for tunnel 3
WORD BeeTrigger_04, BeeDirection_04;	//trigger and direction indicators for tunnel 4
WORD BeeTrigger_05, BeeDirection_05;	//trigger and direction indicators for tunnel 5
WORD BeeTrigger_06, BeeDirection_06;	//trigger and direction indicators for tunnel 6
WORD BeeTrigger_07, BeeDirection_07;	//trigger and direction indicators for tunnel 7
WORD BeeTrigger_08, BeeDirection_08;	//trigger and direction indicators for tunnel 8

//WORD EnabledBeeTrafficCounters;
static WORD BeeClear_01;	//BeeClear flag for tunnel 1
static WORD BeeClear_02;	//BeeClear flag for tunnel 2
static WORD BeeClear_03;	//BeeClear flag for tunnel 3
static WORD BeeClear_04;	//BeeClear flag for tunnel 4
static WORD BeeClear_05;	//BeeClear flag for tunnel 5
static WORD BeeClear_06;	//BeeClear flag for tunnel 6
static WORD BeeClear_07;	//BeeClear flag for tunnel 7
static WORD BeeClear_08;	//BeeClear flag for tunnel 8

BYTE BeeDataLo, BeeDataHi;

//read GPIOA register 09, frist 8 traffic tunnels
	BeeDataLo = MCP23017_readRegister(TrafficPort, TrafficSA, 0x09);
//read GPIOB register 19, next 8 traffic tunnels	
	BeeDataHi = MCP23017_readRegister(TrafficPort, TrafficSA, 0x19);	
	BeeData = (BeeDataHi <<8) | (BeeDataLo);	//get data from all possible tunnels

// Mask off just enabled Bee Traffic Counters
	BeeData = BeeData & EnabledBeeTrafficCounters;	//just use enabled counters

	BeeDirection_01 = BeeData & 0x0100;		//get BeeDirection bit GPB0
	BeeTrigger_01 = BeeData & 0x0200;		//get BeeTrigger GPB1

	BeeDirection_02 = BeeData & 0x0400;		//get BeeDirection bit GPB2
	BeeTrigger_02 = BeeData & 0x0800;		//get BeeTrigger GPB3

	BeeDirection_03 = BeeData & 0x1000;		//get BeeDirection bit GPB4
	BeeTrigger_03 = BeeData & 0x2000;		//get BeeTrigger GPB5

	BeeDirection_04 = BeeData & 0x4000;		//get BeeDirection bit GPB6
	BeeTrigger_04 = BeeData & 0x8000;		//get BeeTrigger GPB7

	BeeDirection_05 = BeeData & 0x0002;		//get BeeDirection bit GPA0
	BeeTrigger_05 = BeeData & 0x0001;		//get BeeTrigger GPA1

	BeeDirection_06 = BeeData & 0x0008;		//get BeeDirection bit GPA2
	BeeTrigger_06 = BeeData & 0x0004;		//get BeeTrigger GPA3

	BeeDirection_07 = BeeData & 0x0020;		//get BeeDirection bit GPA4
	BeeTrigger_07 = BeeData & 0x0010;		//get BeeTrigger GPA5

	BeeDirection_08 = BeeData & 0x0080;		//get BeeDirection bit GPA6
	BeeTrigger_08 = BeeData & 0x0040;		//get BeeTrigger GPA7

//BeeClear is a flag to make sure we don't keep adding counts if the bee is still in the traffic tunnel 
	if (BeeTrigger_01 & BeeClear_01)					//test for valid bee traffic event
	{
	BeeClear_01 = BeeClear_01 & 0xFDFF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_01)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_02 & BeeClear_02)					//test for valid bee traffic event
	{
	BeeClear_02 = BeeClear_02 & 0xF7FF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_02)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_03 & BeeClear_03)					//test for valid bee traffic event
	{
	BeeClear_03 = BeeClear_03 & 0xDFFF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_03)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_04 & BeeClear_04)					//test for valid bee traffic event
	{
	BeeClear_04 = BeeClear_04 & 0x7FFF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_04)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_05 & BeeClear_05)					//test for valid bee traffic event
	{
	BeeClear_05 = BeeClear_05 & 0xFFFE;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_05)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_06 & BeeClear_06)					//test for valid bee traffic event
	{
	BeeClear_06 = BeeClear_06 & 0xFFFB;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_06)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_07 & BeeClear_07)					//test for valid bee traffic event
	{
	BeeClear_07 = BeeClear_07 & 0xFFEF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_07)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

	if (BeeTrigger_08 & BeeClear_08)					//test for valid bee traffic event
	{
	BeeClear_08 = BeeClear_08 & 0xFFBF;		//make the BeeClear flag false, since bee is in tunnel
	// toggle bee direction LED according to the in or out direction of the bee
			if (BeeDirection_08)
			{
				LATDbits.LATD13 ^= 1;		//if BeeDirection true then bee leaving hive
				BeeOutCounter++;
			}
				else
					{
						LATDbits.LATD12 ^= 1;	//if BeeDirection false then bee entering hive
						BeeInCounter++;
					}
	}

// update BeeClear flags
	if (!(BeeTrigger_01 | BeeDirection_01))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_01= BeeClear_01 | 0x0200;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_02 | BeeDirection_02))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_02= BeeClear_02 | 0x0800;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_03 | BeeDirection_03))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_03= BeeClear_03 | 0x2000;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_04 | BeeDirection_04))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_04= BeeClear_04 | 0x8000;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_05 | BeeDirection_05))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_05= BeeClear_05 | 0x0001;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_06 | BeeDirection_06))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_06= BeeClear_06 | 0x0004;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_07 | BeeDirection_07))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_07= BeeClear_07 | 0x0010;		//set the BeeClear flag for this tunnel
	} 

	if (!(BeeTrigger_08 | BeeDirection_08))	//set the clear flag only after bee is out of tunnel
	{
		BeeClear_08= BeeClear_08 | 0x0040;		//set the BeeClear flag for this tunnel
	} 

//==============================
//
//	Need to have flag that is a qualified bee count,i.e. bee is between the two IR detectors to use for the IR thermal detector
//	Also use this to increment the bee traffic in/out counters for valid traffic
return;
}

//==================
//static WORD EnabledBeeTrafficCounters;	//Counters 1-8 configured from web page
//

void ConfigureEnabledBeeTrafficCounters(void)
{
extern FUNCTION_RECORD FunctionModule[FunctionModuleSize];//RAM array that holds current sensor and function status and values

	EnabledBeeTrafficCounters = 0x00;
 
	if ((FunctionModule[ETrafficCounter1].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter1].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x0300;		

	if ((FunctionModule[ETrafficCounter2].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter2].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x0C00;

	if ((FunctionModule[ETrafficCounter3].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter3].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x3000;

	if ((FunctionModule[ETrafficCounter4].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter4].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0xC000;	

	if ((FunctionModule[ETrafficCounter5].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter5].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x0003;		

	if ((FunctionModule[ETrafficCounter6].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter6].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x000C;

	if ((FunctionModule[ETrafficCounter7].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter7].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x0030;

	if ((FunctionModule[ETrafficCounter8].Status.Installed == TRUE)&&		//make sure all conditions are go to enable traffic counter
		 (FunctionModule[ETrafficCounter8].Status.Enabled== TRUE))					
			EnabledBeeTrafficCounters = EnabledBeeTrafficCounters | 0x00C0;	

}

