//======================================================
//
//			M24LC512_lib.c
//			24LC512 EEPROM LIBRARY PROCEDURES
//
//======================================================
//
#include <plib.h>
#include "BMS_HardwareConfig.h"		//Harware configuration for BMS project
//
//*****************************************************
//	I2C2 Issue on PIC32 library function	IdleI2C2();
//
//	Use this code instead of library because of a bug in the library call IdleI2C2();
//
//		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
//		IdleI2C2();		//bug in this library code, use with above line for I2C2
//
//	 IdleI2C1(); works just fine for I2C1
//
//***********************************************************************/
//
//#define SYSCLK	(80000000)
//#define PBCLK  (SYSCLK/2)
//#define Fsck	50000
//#define BRG_VAL 	((PBCLK/2/Fsck)-2)
/**************************************************************
 * USE 7-bit slave addresse in function calls
 *************************************************************/
unsigned char Eeprom_Read(unsigned char I2Cport, unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte);
void Eeprom_Write(unsigned char I2Cport, unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte);

unsigned char Eeprom_I2C1_Read(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte);
void Eeprom_I2C1_Write(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte);

unsigned char Eeprom_I2C2_Read(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte);
void Eeprom_I2C2_Write(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte);

//==============================================================================
void Eeprom_Write(unsigned char I2Cport, unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte)
{
	if (I2Cport == 1)
	{
		Eeprom_I2C1_Write(DevAddress, HighAdrByte, LowAdrByte, DataByte);
	}
		else
		{
			if (I2Cport == 2)
			{
				Eeprom_I2C2_Write(DevAddress, HighAdrByte, LowAdrByte, DataByte);
			}
		}
}

unsigned char Eeprom_Read(unsigned char I2Cport, unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte)
{
	if (I2Cport == 1)
	{
		Eeprom_I2C1_Read(DevAddress, HighAdrByte, LowAdrByte);
	}
		else
		{
			if (I2Cport == 2)
			{
				Eeprom_I2C2_Read(DevAddress, HighAdrByte, LowAdrByte);
			}
		}
}

//======================================================================
//
//		I2C1 port
//
//======================================================================
// Send Data to eeprom to program one location
void Eeprom_I2C1_Write(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte)
{
	char i2cData[4];
	int DataSz;
	unsigned char Data;
	i2cData[0] = (DevAddress << 1) | 0;	//EEPROM Device Address and WR Command
	i2cData[1] = HighAdrByte;	//eeprom location to program (high address byte)
	i2cData[2] = LowAdrByte;	//eeprom location to program (low address byte)
	i2cData[3] = DataByte;	//data to write
	DataSz = 4;

	StartI2C1();	//Send the Start Bit
	IdleI2C1();		//Wait to complete

	int Index = 0;
	while( DataSz )
	{
		MasterWriteI2C1( i2cData[Index++] );
		IdleI2C1();		//Wait to complete

		DataSz--;

		//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
		if( I2C1STATbits.ACKSTAT )
		{
		Data = Data +1;
			break;
		}
	}

	StopI2C1();	//Send the Stop condition
	IdleI2C1();	//Wait to complete
	// wait for EEPROM to complete write process. poll the ack status
	while(1)
	{
 		i2c_wait(10);	//this statment may not be required

		StartI2C1();	//Send the Start Bit
		IdleI2C1();		//Wait to complete

		MasterWriteI2C1( i2cData[0] );
		IdleI2C1();		//Wait to complete

		if( I2C1STATbits.ACKSTAT == 0 )	//EEPROM has acknowledged
		{
			StopI2C1();	//Send the Stop condition
			IdleI2C1();	//Wait to complete
			break;
		}

		StopI2C1();	//Send the Stop condition
		IdleI2C1();	//Wait to complete
	}
}



unsigned char Eeprom_I2C1_Read(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte)
{
	char i2cData[3];
	int DataSz;
	int Index;
	unsigned char Data;
	i2cData[0] = (DevAddress << 1) | 0;	//EEPROM Device Address and WR Command
	i2cData[1] = HighAdrByte;	//EEPROM location to program (high address byte)
	i2cData[2] = LowAdrByte;	//EEPROM location to program (low address byte)
	DataSz = 3;

	StartI2C1();	//Send the Start Bit
	IdleI2C1();		//Wait to complete

	//send the address to read from the serial eeprom
	Index = 0;
	while( DataSz )
	{
		MasterWriteI2C1( i2cData[Index++] );
		IdleI2C1();		//Wait to complete

		DataSz--;

		//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
		if( I2C1STATbits.ACKSTAT )
		{
		Data = Data +1;
			break;
		}
	}

	//send a start sequence again
	RestartI2C1();	//Send the Restart condition

	//wait for this bit to go back to zero
	IdleI2C1();	//Wait to complete

	MasterWriteI2C1( (DevAddress << 1) | 1 ); //transmit read command
	IdleI2C1();		//Wait to complete

	Data = MasterReadI2C1();

	StopI2C1();	//Send the Stop condition
	IdleI2C1();	//Wait to complete
	return Data;
}
//

//======================================================================
//
//		I2C2 port
//
//======================================================================

// Send Data to eeprom to program one location
void Eeprom_I2C2_Write(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte, unsigned char DataByte)
{
	char i2cData[4];
	int DataSz;
	unsigned char Data;
	
	i2cData[0] = (DevAddress << 1) | 0;	//EEPROM Device Address and WR Command
	i2cData[1] = HighAdrByte;	//eeprom location to program (high address byte)
	i2cData[2] = LowAdrByte;	//eeprom location to program (low address byte)
	i2cData[3] = DataByte;	//data to write
	DataSz = 4;

	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	int Index = 0;
	while( DataSz )
		{
		MasterWriteI2C2( i2cData[Index++] );
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2

		DataSz--;

		//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
		if( I2C2STATbits.ACKSTAT )
			{
		Data = Data +1;
			break;
			}
		}

	StopI2C2();	//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	// wait for EEPROM to complete write process. poll the ack status
	while(1)
	{
 		i2c_wait(10);	//this statment may not be required

		StartI2C2();	//Send the Start Bit
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2

		MasterWriteI2C2( i2cData[0] );
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2

		if( I2C2STATbits.ACKSTAT == 0 )	//EEPROM has acknowledged
		{
		StopI2C2();	//Send the Stop condition
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2
		break;
		}

		StopI2C2();	//Send the Stop condition
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2
	}
}



unsigned char Eeprom_I2C2_Read(unsigned char DevAddress, unsigned char HighAdrByte, unsigned char LowAdrByte)
{
	char i2cData[3];
	int DataSz;
	int Index;
	unsigned char Data;
	i2cData[0] = (DevAddress << 1) | 0;	//EEPROM Device Address and WR Command
	i2cData[1] = HighAdrByte;	//EEPROM location to program (high address byte)
	i2cData[2] = LowAdrByte;	//EEPROM location to program (low address byte)
	DataSz = 3;

	StartI2C2();	//Send the Start Bit
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	//send the address to read from the serial eeprom
	Index = 0;
	while( DataSz )
	{
		MasterWriteI2C2( i2cData[Index++] );
		while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
		IdleI2C2();		//bug in this library code, use with above line for I2C2

		DataSz--;

		//ACKSTAT is 0 when slave acknowledge. if 1 then slave has not acknowledge the data.
		if( I2C2STATbits.ACKSTAT )
		{
		Data = Data +1;
			break;
		}
	}

	//send a start sequence again
	RestartI2C2();	//Send the Restart condition

	//wait for this bit to go back to zero
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	MasterWriteI2C2( (DevAddress << 1) | 1 ); //transmit read command
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2

	Data = MasterReadI2C2();

	StopI2C2();	//Send the Stop condition
	while(I2C2CONbits.RSEN) {};  //wait for RSEN to go low 
	IdleI2C2();		//bug in this library code, use with above line for I2C2
	return Data;
}

